FUNCTION PROJECTION, P, A, E, IRAD, OMEGA_PERI_RAD, OMEGA_NODE_RAD, T0, OBSTIME, NGEN, DIST

;+
; NAME: PROJECTION

; PURPOSE: this function evaluates the parameter of the projected orbit of a
;          planet, at a given time
;
; CALLING SEQUENCE:
;                 result=PROJECTION(p, a, e, irad, omega_peri_rad, $
;                                   omega_node_rad, t0, obstime, ngen, dpc)
; INPUTS:
;        P - orbital period (days)
;        A - semimajor axis (AU)
;        E - eccentricity
;        IRAD - inclination (rad)
;        OMEGA_PERI_RAD - longitude of periastron (rad)
;        OMEGA_NODE_RAD - longitude of node (rad)
;        T0 - time of periastron passage
;        OBSTIME - time of observation
;        NGEN - number of generated planets
;        DIST - distance (pc)

;
; OPTIONAL INPUT KEYWORDS:
;
; OUTPUTS:
;        the output is a structure, that contains the following elements
;        result.rho          =  projected separation (AU)
;        result.rho_arcsec   =  projected separation (arcsec)
;        result.x2           =  projected x coordinate (AU)
;        result.y2           =  projected y coordinate (AU)
;        result.frac_ill     =  illuminated fraction of the planet surface


; EXAMPLES:
;
; RESTRICTIONS:
;
; PROCEDURES CALLED
;
;
; REVISION HISTORY:
;       Written: S. Desidera 2006         
;       Modified: M. Bonavita 2009. Introduced the output structure, 
;                 and the evaluation of the illuminated fraction. 
;                 Fixed the bugs in the computation of the x and y 
;                 position on the projected orbit     
;-




nu=DBLARR(ngen)
r=DBLARR(ngen)
rho=DBLARR(ngen)
rho2=DBLARR(ngen)
zeta=DBLARR(ngen)

theta=DBLARR(ngen)
theta2=DBLARR(ngen)
theta3=DBLARR(ngen)
theta4=DBLARR(ngen)

an_m=DBLARR(ngen)
an_e=DBLARR(ngen)
an_e2=DBLARR(ngen)


x1=DBLARR(ngen)
x2=DBLARR(ngen)

y1=DBLARR(ngen)
Y2=DBLARR(ngen)

uuu=DBLARR(ngen)

phase_angle=DBLARR(ngen)

frac_ill=DBLARR(ngen)


;;;;;;;an_m2=RANDOMU(seed, ngen)

FOR j=0L,ngen-1 DO BEGIN

 IF e[j] eq 0 then begin

;   t0[j]=obstime[j]
   omega_peri_rad[j]=0
 END;IF
    ; Calculate the approximate eccentric anomaly, E1, via the mean 
    ; anomoly, M.
    ; (from Heintz DW, "Double stars", Reidel, 1978)
    ;
    ; M=2.D*!dpi*( (HJD-T)/P MOD 1.)
    ; E1=M + e*sin(M)  + ((e^2)*sin(2.0D*M)/2.0D)

if t0[j] eq obstime[j] then t0[j]=t0[j]+1
uuu[j]=( (obstime[j]-t0[j])/P[j] MOD 1.)
;print, 'uuu', uuu(j)

    M=2.D*!dpi*( (obstime[j]-t0[j])/P[j] )

    E1=M + e[j]*SIN(M)  + ((e[j]^2)*SIN(2.0D*M)/2.0D)
 
    an_m[j]=m   
    an_e[j]=e1  

    ; Now refine this estimate using  formulae given by Reidel.

 ii=0
if e[j] eq 0 then goto, circular
 repeat begin
 ii=ii+1
  E0=E1 
  M0 = E0 - e[j]*sin(E0)
  E1 = E0 + (M-M0)/(1.0 - e[j]*cos(E0))
  TEST = max(abs((E1-E0)/E1[where (E1 ne 0.0)]))
  endrep until TEST lt 1D-8

;print, e1*!radeg
circular:
an_e2[j]=e1



A1=a(j)*((COS(omega_peri_rad(j))*COS(omega_node_rad(j)))-(SIN(omega_peri_rad(j))*SIN(omega_node_rad(j))*COS(irad(j))))
B1=a(j)*((COS(omega_peri_rad(j))*SIN(omega_node_rad(j)))+(SIN(omega_peri_rad(j))*COS(omega_node_rad(j))*COS(irad(j))))
F1=a(j)*((-1*SIN(omega_peri_rad(j))*COS(omega_node_rad(j)))-(COS(omega_peri_rad(j))*SIN(omega_node_rad(j))*COS(irad(j))))
G1=a(j)*((-1*SIN(omega_peri_rad(j))*SIN(omega_node_rad(j)))+(COS(omega_peri_rad(j))*COS(omega_node_rad(j))*COS(irad(j))))

x1(j)=COS(an_e2[j])-e(j)
y1(j)=SQRT(1-(e(j)^2))*SIN(an_e2[j])
x2(j)=A1*x1(j)+F1*y1(j)
y2(j)=B1*x1(j)+G1*y1(j)




;
; Now calculate nu
;
nu[j]=2.0D*ATAN( SQRT((1.0D + e[j])/(1.D - e[j]))*TAN(E1/2.0D) )
; nu=nu+((nu<0D)*(2D*!dpi))
;

r[j]=a[j]*(1-(e[j])^2)/(1+(e[j])*COS(nu[j]))

rho(j)=SQRT((x2(j)^2)+(y2(j)^2))

ENDFOR 

theta=ATAN((TAN(nu+omega_peri_rad)*COS(irad)+TAN(omega_node_rad))/(1-TAN(nu+omega_peri_rad)*COS(irad)*TAN(omega_node_rad)))



nudeg=!radeg*nu



theta_deg=theta*!radeg
theta2_deg=theta2*!radeg
theta3_deg=theta3*!radeg
theta4_deg=theta4*!radeg

rho_arcsec=rho/dist

phi=(an_m)/(2.0D*!dpi)  ;mean anomaly

frac_ill=0.5D*(1-SIN(irad)*SIN(2.0D*!dpi*phi))
zeta=r*sin(nu+omega_peri_rad)
beta=ACOS(zeta/SQRT(zeta^2+rho^2))
frac_ill=(SIN(beta)+(!dpi-beta)*COS(beta))/!dpi


i=irad*!radeg

w=WHERE(r GT 12.0 AND i GT 80.0 AND i LT 100.0, cw)

IF cw NE 0 THEN BEGIN

w1=frac_ill(w)
w2=rho(w)


ENDIF

alpha=(!dpi/2.D)-nu
;help, alpha

phialpha=0.5D * (1.0D + SIN(alpha))

phialpha=(SIN(alpha)+(!dpi-alpha)*COS(alpha))/!dpi


out_struc=CREATE_STRUCT('rho', rho, $
                        'rho_arcsec', rho_arcsec, $
                        'x2', x2,$
                        'y2', y2,$
                        'frac_ill', frac_ill)





RETURN, out_struc

END
